"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildProject = exports.buildTarget = void 0;
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const config_1 = require("../config");
const utils_1 = require("../utils");
const logger_1 = __importDefault(require("../utils/logger"));
exports.buildTarget = async (config, target, { outDir = '', _require = require }) => {
    const resolvedTargetModule = utils_1.getResolvedTargetModule(target);
    logger_1.default('resolved module:', resolvedTargetModule);
    if (!resolvedTargetModule) {
        logger_1.default('No module found for target:', target);
        return;
    }
    const targetModule = _require(resolvedTargetModule);
    const generateExtensionInfo = targetModule.generateExtensionInfo;
    // TODO: Build target steps: beforeBuild:target, afterBuild:target, preCompile:target, postCompile:target
    // Compile templates with config data
    const extensionInfo = await generateExtensionInfo(config);
    logger_1.default(`${target} extension info:`, extensionInfo);
    const extensionOutDir = path_1.resolve(outDir, `${target}-files`);
    logger_1.default('Resolved output directory:', outDir);
    logger_1.default('Removing output directory..');
    await fs_extra_1.remove(extensionOutDir);
    const includedFiles = await utils_1.getFiles(config.include, {
        ignore: config.exclude,
        onlyFiles: false,
        expandDirectories: false,
        absolute: true,
        cwd: path_1.resolve(config.rootDir),
    });
    logger_1.default('Included files:', includedFiles);
    logger_1.default('Copying included files to output directory..');
    for (const file of includedFiles) {
        await fs_extra_1.copy(file, path_1.resolve(extensionOutDir, path_1.basename(file)), {
            filter: (src) => {
                return !config.exclude.find(excludeRegex => {
                    return new RegExp(excludeRegex).test(src);
                });
            }
        });
    }
    logger_1.default('Copying manifest file to output directory..');
    const manifestOutputPath = path_1.resolve(extensionOutDir, extensionInfo.fileName);
    fs_extra_1.outputFile(manifestOutputPath, extensionInfo.content, 'utf8');
    const compileExtension = targetModule.compileExtension;
    if (compileExtension) {
        if (config.beforeCompile) {
            const resolvedBeforeCompileModule = utils_1.getResolvedModule(path_1.resolve(process.cwd(), config.beforeCompile));
            if (resolvedBeforeCompileModule) {
                logger_1.default('Executing beforeCompile script..', resolvedBeforeCompileModule);
                const beforeCompileModule = _require(resolvedBeforeCompileModule);
                await beforeCompileModule({
                    config,
                    extensionFilesDir: extensionOutDir,
                });
            }
        }
        logger_1.default('Compiling extension..');
        await compileExtension({
            config,
            extensionFilesDir: extensionOutDir,
            extensionBuildOutputDir: path_1.resolve(outDir, `${target}-build`),
        });
    }
    logger_1.default('Build completed.');
};
exports.buildProject = async ({ configPath = '', _require = require } = {}) => {
    logger_1.default('Current working directory:', process.cwd());
    const config = await config_1.getConfig(configPath);
    logger_1.default('Config:', config);
    const outDir = path_1.resolve(config.rootDir, config.outDir);
    logger_1.default('Resolved output directory:', outDir);
    logger_1.default('Removing output directory..');
    await fs_extra_1.remove(outDir);
    logger_1.default(`Building for ${config.targets.length} targets:`, config.targets);
    for (const target of config.targets) {
        let targetConfig = config;
        if (config.targetOptions && config.targetOptions[target]) {
            targetConfig = Object.assign(Object.assign({}, config), config.targetOptions[target]);
        }
        await exports.buildTarget(targetConfig, target, { outDir, _require });
    }
};
//# sourceMappingURL=build.js.map