"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileExtension = exports.generateExtensionInfo = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const archiver_1 = __importDefault(require("archiver"));
const fs_extra_1 = require("fs-extra");
const logger_1 = __importDefault(require("../utils/logger"));
const buildBrowserAction = (config) => {
    var _a;
    if ((_a = config.manifestOptions) === null || _a === void 0 ? void 0 : _a.browser_action) {
        return {
            default_icon: config.manifestOptions.browser_action.default_icon,
        };
    }
    return {};
};
const buildOptionsUi = (config) => {
    var _a;
    if ((_a = config.manifestOptions) === null || _a === void 0 ? void 0 : _a.options_ui) {
        return {
            page: config.manifestOptions.options_ui.page,
            open_in_tab: config.manifestOptions.options_ui.open_in_tab,
        };
    }
    return undefined;
};
const buildExtensionData = (config) => {
    if (!config.manifestOptions) {
        return null;
    }
    return {
        manifest_version: 2,
        name: config.manifestOptions.name,
        version: config.manifestOptions.version,
        short_name: config.manifestOptions.short_name,
        description: config.manifestOptions.description,
        author: config.manifestOptions.author,
        icons: config.manifestOptions.icons,
        browser_action: buildBrowserAction(config),
        permissions: config.manifestOptions.permissions,
        content_security_policy: config.manifestOptions.content_security_policy,
        background: config.manifestOptions.background,
        options_ui: buildOptionsUi(config),
        offline_enabled: config.manifestOptions.offline_enabled,
        chrome_settings_overrides: config.manifestOptions.chrome_settings_overrides,
        chrome_ui_overrides: config.manifestOptions.chrome_ui_overrides,
        chrome_url_overrides: config.manifestOptions.chrome_url_overrides,
        commands: config.manifestOptions.commands,
        content_scripts: config.manifestOptions.content_scripts,
        devtools_page: config.manifestOptions.devtools_page,
        homepage_url: config.manifestOptions.homepage_url,
        incognito: config.manifestOptions.incognito,
        minimum_chrome_version: config.manifestOptions.minimum_chrome_version,
        omnibox: config.manifestOptions.omnibox,
        storage: config.manifestOptions.storage,
        web_accessible_resources: config.manifestOptions.web_accessible_resources,
    };
};
exports.generateExtensionInfo = async (config) => {
    const extensionData = buildExtensionData(config);
    return {
        content: extensionData ? JSON.stringify(extensionData, null, 2) : "",
        fileName: "manifest.json",
        fileType: "json",
    };
};
exports.compileExtension = async (opts) => {
    var _a;
    const outputPath = path_1.default.resolve(opts.extensionBuildOutputDir, (_a = opts.config.outFile) !== null && _a !== void 0 ? _a : "chrome.zip");
    await fs_extra_1.ensureFile(outputPath);
    const output = fs_1.default.createWriteStream(outputPath);
    return new Promise((resolve, reject) => {
        // zip extension files
        const archive = archiver_1.default("zip");
        // listen for all archive data to be written
        // 'close' event is fired only when a file descriptor is involved
        output.on("close", () => {
            // log(archive.pointer() + ' total bytes');
            // log('archiver has been finalized and the output file descriptor has closed.');
            return resolve(true);
        });
        // This event is fired when the data source is drained no matter what was the data source.
        // It is not part of this library but rather from the NodeJS Stream API.
        // @see: https://nodejs.org/api/stream.html#stream_event_end
        output.on("end", () => {
            logger_1.default("Data has been drained");
            return resolve(true);
        });
        // good practice to catch warnings (ie stat failures and other non-blocking errors)
        archive.on("warning", (err) => {
            if (err.code === "ENOENT") {
                // log warning
                logger_1.default(err);
            }
            else {
                // throw error
                return reject(err);
            }
        });
        // good practice to catch this error explicitly
        archive.on("error", (err) => {
            return reject(err);
        });
        // pipe archive data to the file
        archive.pipe(output);
        // append files from a sub-directory, putting its contents at the root of archive
        archive.directory(opts.extensionFilesDir, false);
        // finalize the archive (ie we are done appending files but streams have to finish yet)
        // 'close', 'end' or 'finish' may be fired right after calling this method so register to them beforehand
        archive.finalize();
    });
};
exports.default = exports.generateExtensionInfo;
//# sourceMappingURL=chrome.js.map